<?php

namespace InSegment\WorkersFramework\Queues;

use InSegment\WorkersFramework\Interfaces\JobConfigInterface;
use InSegment\WorkersFramework\Interfaces\JobListInterface;
use InSegment\WorkersFramework\Models\JobConfig;

class JobList implements JobListInterface
{
    
    /**
     * Known jobs
     *
     * @var \InSegment\WorkersFramework\Interfaces\JobConfigInterface[]
     */
    protected $jobs = [];
    
    /**
     * Get nodes a job can be done on
     *
     * @var array
     */
    protected $jobNodes = [];
    
    /**
     * Constructor
     * 
     * @param array $list
     */
    public function __construct(array $list)
    {
        foreach ($list as $jobName => $jobConfig) {
            if (!isset($jobConfig['class'])) {
                throw new \Exception("Class is not defined for job: {$jobName}");
            }
            
            $jobClass = $jobConfig['class'];
            
            if (!class_exists($jobClass)) {
                throw new \Exception("Jobs config checking: Class not found: {$jobClass} for job {$jobName}");
            }
            
            $this->addJob($jobName, new JobConfig($jobName, $jobConfig));
        }
    }
    
    /**
     * {@inheritDoc}
     */
    public function getJob(string $jobName): ?JobConfigInterface
    {
        return $this->jobs[$jobName] ?? null;
    }
    
    /**
     * {@inheritDoc}
     */
    public function getJobWorkers(string $jobName): ?array
    {
        return $this->jobWorkers[$jobName] ?? null;
    }
    
    /**
     * Add job to the list
     * 
     * @param string $name
     * @param \InSegment\WorkersFramework\Interfaces\JobConfigInterface $job
     * @return null
     */
    protected function addJob(string $name, JobConfigInterface $job)
    {
        $this->jobs[$name] = $job;
    }
    
}
