<?php

namespace InSegment\WorkersFramework\Services;

use InSegment\WorkersFramework\Interfaces\JobListInterface;
use InSegment\WorkersFramework\Queues\UnpublishedMessage;
use InSegment\WorkersFramework\Interfaces\QueueInterface;
use InSegment\WorkersFramework\Interfaces\QueueMessageInterface;
use InSegment\WorkersFramework\Models\JobHeaders;

class MainConsumer extends AbstractConsumer
{
    
    /**
     * Distributor instance
     *
     * @var \InSegment\WorkersFramework\Services\Distributor
     */
    protected $distrubutor;
    
    /**
     * JobList instance
     *
     * @var \InSegment\WorkersFramework\Interfaces\JobListInterface
     */
    protected $jobList;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Services\QueueInterface $queue
     * @param \InSegment\WorkersFramework\Services\Distributor $distributor
     * @param  \InSegment\WorkersFramework\Interfaces\JobListInterface $jobList
     */
    public function __construct(QueueInterface $queue, Distributor $distributor, JobListInterface $jobList)
    {
        parent::__construct($queue);
        $this->distrubutor = $distributor;
        $this->jobList = $jobList;
    }
    
    /**
     * Move message from the main queue to the worker queue and acknowledge the original one
     * 
     * @param \InSegment\WorkersFramework\Interfaces\QueueMessageInterface $message
     * @return mixed
     */
    public function receiveMessage(QueueMessageInterface $message)
    {
        $headers = JobHeaders::extend($message->getHeaders());
        $headersValidation = $headers->validate();
        
        if (!$headersValidation->isValid()) {
            $printNotFoundFields = implode(', ', $headersValidation->getInvalidFields());
            echo "Job header mandatory fields not found: {$printNotFoundFields}, message will be ignored\n";
            return $message->acknowlege();
        }
        
        $jobName = $headers->getJobName();
        $job = $this->jobList->getJob($jobName);
        
        if ($job === null) {
            echo "The configuration doesn't have class mapping for job named {$jobName}, message will be ignored\n";
            return $message->acknowlege();
        }
        
        $worker = $this->distrubutor->findBestWorkerFor($job);
        $queue = $this->distrubutor->getQueueForWorker($worker);
        
        $newMessage = new UnpublishedMessage($queue, $message->getContents(), $message->getHeaders());
        
        if ($this->trigger('publishing', $worker, $job, $newMessage) !== false) {
            return $newMessage->send()->then(function() use ($message, $worker, $job, $newMessage) {
                $this->trigger('published', $worker, $job, $newMessage);
                return $message->acknowlege();
            });
        } else {
            return $message->acknowlege();
        }
    }

}
